@@ -1,7 +1,18 @@
 /*
- * This Source Code is subject to the terms of the Mozilla Public License
- * version 2.0 (the "License"). You can obtain a copy of the License at
- * http://mozilla.org/MPL/2.0/.
+ * This file is part of Adblock Plus <https://adblockplus.org/>,
+ * Copyright (C) 2006-2017 eyeo GmbH
+ *
+ * Adblock Plus is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * Adblock Plus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -15,19 +26,20 @@ var FilterActions =
    */
   init: function()
   {
-    let me = this;
-    this.treeElement.parentNode.addEventListener("keypress", function(event)
-    {
-      me.keyPress(event);
-    }, true);
+    this.treeElement.parentNode.addEventListener("keydown", this.keyDown.bind(this), true);
     this.treeElement.view = FilterView;
 
-    this.treeElement.inputField.addEventListener("keypress", function(event)
+    // Work around https://bugzilla.mozilla.org/show_bug.cgi?id=777832, don't
+    // allow the tree to receive keypress/keydown events triggered by cursor
+    // keys pressed in the editor, it will call preventDefault() on them.
+    let propagationStopper = function(event)
     {
-      // Prevent the tree from capturing cursor keys pressed in the input field
       if (event.keyCode >= event.DOM_VK_PAGE_UP && event.keyCode <= event.DOM_VK_DOWN)
         event.stopPropagation();
-    }, false);
+    };
+
+    this.treeElement.inputField.addEventListener("keypress", propagationStopper, false);
+    this.treeElement.inputField.addEventListener("keydown", propagationStopper, false);
 
     // Create a copy of the view menu
     function fixId(node, newId)
@@ -89,9 +101,9 @@ var FilterActions =
   fillActionsPopup: function()
   {
     let editable = FilterView.editable;
-    let items = FilterView.selectedItems.filter(function(i) !i.filter.dummy);
-    items.sort(function(entry1, entry2) entry1.index - entry2.index);
-    let activeItems = items.filter(function(i) i.filter instanceof ActiveFilter);
+    let items = FilterView.selectedItems.filter(i => !i.filter.dummy);
+    items.sort((entry1, entry2) => entry1.index - entry2.index);
+    let activeItems = items.filter(i => i.filter instanceof ActiveFilter);
 
     E("filters-edit-command").setAttribute("disabled", !editable || !items.length);
     E("filters-delete-command").setAttribute("disabled", !editable || !items.length);
@@ -130,7 +142,7 @@ var FilterActions =
     if (this.treeElement.editingColumn)
       return;
 
-    let items = FilterView.selectedItems.filter(function(i) i.filter instanceof ActiveFilter);
+    let items = FilterView.selectedItems.filter(i => i.filter instanceof ActiveFilter);
     if (items.length)
     {
       FilterView.boxObject.beginUpdateBatch();
@@ -193,7 +205,7 @@ var FilterActions =
   deleteItems: function(/**Array*/ items)
   {
     let oldIndex = FilterView.selection.currentIndex;
-    items.sort(function(entry1, entry2) entry2.index - entry1.index);
+    items.sort((entry1, entry2) => entry2.index - entry1.index);
 
     for (let i = 0; i < items.length; i++)
       FilterStorage.removeFilter(items[i].filter, FilterView._subscription, items[i].index);
@@ -224,9 +236,9 @@ var FilterActions =
     if (this.treeElement.editingColumn)
       return;
 
-    let items = FilterView.selectedItems.filter(function(i) i.filter instanceof ActiveFilter);
+    let items = FilterView.selectedItems.filter(i => i.filter instanceof ActiveFilter);
     if (items.length)
-      FilterStorage.resetHitCounts(items.map(function(i) i.filter));
+      FilterStorage.resetHitCounts(items.map(i => i.filter));
   },
 
   /**
@@ -241,7 +253,7 @@ var FilterActions =
 
     if (offset < 0)
     {
-      items.sort(function(entry1, entry2) entry1.index - entry2.index);
+      items.sort((entry1, entry2) => entry1.index - entry2.index);
       let position = items[0].index + offset;
       if (position < 0)
         return;
@@ -252,7 +264,7 @@ var FilterActions =
     }
     else if (offset > 0)
     {
-      items.sort(function(entry1, entry2) entry2.index - entry1.index);
+      items.sort((entry1, entry2) => entry2.index - entry1.index);
       let position = items[0].index + offset;
       if (position >= FilterView.rowCount)
         return;
@@ -327,10 +339,17 @@ var FilterActions =
       while (box.firstChild)
         box.removeChild(box.firstChild);
 
-      for (var i = 0; i < text.length; i += 80)
+      let lines = text.match(/.{1,80}/g);
+      if (lines.length > 7)
       {
-        var description = document.createElement("description");
-        description.setAttribute("value", text.substr(i, 80));
+        // Text is too long to display in full so we cut out the middle part
+        lines = lines.slice(0,3).concat("\u2026", lines.slice(-3));
+      }
+
+      for (let line of lines)
+      {
+        let description = document.createElement("description");
+        description.setAttribute("value", line);
         box.appendChild(description);
       }
     }
@@ -347,7 +366,7 @@ var FilterActions =
 
     E("tooltip-additional").hidden = false;
     if (item.filter instanceof InvalidFilter && item.filter.reason)
-      E("tooltip-additional").textContent = item.filter.reason;
+      E("tooltip-additional").textContent = Utils.getString(item.filter.reason);
     else if (item.filter instanceof RegExpFilter && defaultMatcher.isSlowFilter(item.filter))
       E("tooltip-additional").textContent = Utils.getString("filter_regexp_tooltip");
     else
@@ -357,7 +376,7 @@ var FilterActions =
   /**
    * Called whenever a key is pressed on the list.
    */
-  keyPress: function(/**Event*/ event)
+  keyDown: function(/**Event*/ event)
   {
     if (event.target != E("filtersTree"))
       return;
@@ -370,7 +389,7 @@ var FilterActions =
     if (event.metaKey)
       modifiers |= SubscriptionActions._metaMask;
 
-    if (event.charCode == " ".charCodeAt(0) && modifiers == 0 && !E("col-enabled").hidden)
+    if (event.keyCode == Ci.nsIDOMKeyEvent.DOM_VK_SPACE && modifiers == 0 && !E("col-enabled").hidden)
       this.selectionToggleDisabled();
     else if (event.keyCode == Ci.nsIDOMKeyEvent.DOM_VK_UP && modifiers == SubscriptionActions._accelMask)
     {
@@ -396,8 +415,8 @@ var FilterActions =
     if (!items.length)
       return;
 
-    items.sort(function(entry1, entry2) entry1.index - entry2.index);
-    let text = items.map(function(i) i.filter.text).join(IO.lineBreak);
+    items.sort((entry1, entry2) => entry1.index - entry2.index);
+    let text = items.map(i => i.filter.text).join(IO.lineBreak);
     Utils.clipboardHelper.copyString(text);
 
     if (!keep && FilterView.editable && !this.treeElement.editingColumn)
@@ -433,9 +452,12 @@ var FilterActions =
     let lines = data.replace(/\r/g, "").split("\n");
     for (let i = 0; i < lines.length; i++)
     {
-      let filter = Filter.fromText(lines[i]);
-      if (filter)
+      let line = Filter.normalize(lines[i]);
+      if (line)
+      {
+        let filter = Filter.fromText(line);
         FilterStorage.addFilter(filter, FilterView._subscription, position++);
+      }
     }
   },
 
@@ -450,8 +472,8 @@ var FilterActions =
     if (!items.length)
       return;
 
-    items.sort(function(entry1, entry2) entry1.index - entry2.index);
-    event.dataTransfer.setData("text/plain", items.map(function(i) i.filter.text).join(IO.lineBreak));
+    items.sort((entry1, entry2) => entry1.index - entry2.index);
+    event.dataTransfer.setData("text/plain", items.map(i => i.filter.text).join(IO.lineBreak));
     this.dragItems = items;
     event.stopPropagation();
   },
@@ -496,9 +518,12 @@ var FilterActions =
         let lines = data.replace(/\r/g, "").split("\n");
         for (let i = 0; i < lines.length; i++)
         {
-          let filter = Filter.fromText(lines[i]);
-          if (filter)
+          let line = Filter.normalize(lines[i]);
+          if (line)
+          {
+            let filter = Filter.fromText(line);
             FilterStorage.addFilter(filter, FilterView._subscription, newPosition++);
+          }
         }
       }
       return;
