@@ -1,7 +1,18 @@
 /*
- * This Source Code is subject to the terms of the Mozilla Public License
- * version 2.0 (the "License"). You can obtain a copy of the License at
- * http://mozilla.org/MPL/2.0/.
+ * This file is part of Adblock Plus <https://adblockplus.org/>,
+ * Copyright (C) 2006-2017 eyeo GmbH
+ *
+ * Adblock Plus is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * Adblock Plus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
@@ -18,6 +29,22 @@ var FilterView =
    */
   init: function()
   {
+    // "Manually" remove access key for col-slow tooltip, Utils.splitAllLabels()
+    // won't do it.
+    let slowColumn = document.getElementById("col-slow");
+    if (slowColumn)
+    {
+      for (let attr of ["display", "tooltiptext"])
+      {
+        let value = slowColumn.getAttribute(attr);
+        if (!value)
+          continue;
+        let [label, accessKey] = Utils.splitLabel(value);
+        if (label != value)
+          slowColumn.setAttribute(attr, label);
+      }
+    }
+
     if (this.sortProcs)
       return;
 
@@ -435,14 +462,22 @@ var FilterView =
 
   /**
    * Selects a row in the tree and makes sure it is visible.
+   * @param {number} row
+   *   row index
+   * @param {boolean} [scrollToTop]
+   *   if true, the selected row should become the top row of the list if
+   *   possible, otherwise the list is only scrolled if the row isn't visible.
    */
-  selectRow: function(row)
+  selectRow: function(row, scrollToTop)
   {
     if (this.selection)
     {
       row = Math.min(Math.max(row, 0), this.data.length - 1);
       this.selection.select(row);
-      this.boxObject.ensureRowIsVisible(row);
+      if (scrollToTop)
+        this.boxObject.scrollToRow(row);
+      else
+        this.boxObject.ensureRowIsVisible(row);
     }
   },
 
@@ -462,7 +497,7 @@ var FilterView =
     }
     if (index >= 0)
     {
-      this.selectRow(index);
+      this.selectRow(index, true);
       this.treeElement.focus();
     }
   },
@@ -475,7 +510,7 @@ var FilterView =
     let oldCount = this.rowCount;
     if (this._subscription && this._subscription.filters.length)
     {
-      this.data = this._subscription.filters.map(function(f, i) ({index: i, filter: f}));
+      this.data = this._subscription.filters.map((f, i) => ({index: i, filter: f}));
       if (this.sortProc)
       {
         // Hide comments in the list, they should be sorted like the filter following them
@@ -606,35 +641,41 @@ var FilterView =
 
   setTree: function(boxObject)
   {
+    if (!boxObject)
+      return;
+
     this.init();
     this.boxObject = boxObject;
 
-    if (this.boxObject)
+    this.noGroupDummy = {index: 0, filter: {text: this.boxObject.treeBody.getAttribute("noGroupText"), dummy: true}};
+    this.noFiltersDummy = {index: 0, filter: {text: this.boxObject.treeBody.getAttribute("noFiltersText"), dummy: true}};
+    this.editDummy = {filter: {text: ""}};
+
+    let atomService = Cc["@mozilla.org/atom-service;1"].getService(Ci.nsIAtomService);
+    let stringAtoms = ["col-filter", "col-enabled", "col-hitcount", "col-lasthit", "type-invalid", "type-comment", "type-blocking", "type-whitelist", "type-elemhide", "type-elemhideexception", "type-elemhideemulation"];
+    let boolAtoms = ["selected", "dummy", "slow", "disabled"];
+
+    this.atoms = {};
+    for (let atom of stringAtoms)
+      this.atoms[atom] = atomService.getAtom(atom);
+    for (let atom of boolAtoms)
     {
-      this.noGroupDummy = {index: 0, filter: {text: this.boxObject.treeBody.getAttribute("noGroupText"), dummy: true}};
-      this.noFiltersDummy = {index: 0, filter: {text: this.boxObject.treeBody.getAttribute("noFiltersText"), dummy: true}};
-      this.editDummy = {filter: {text: ""}};
-
-      let atomService = Cc["@mozilla.org/atom-service;1"].getService(Ci.nsIAtomService);
-      let stringAtoms = ["col-filter", "col-enabled", "col-hitcount", "col-lasthit", "type-comment", "type-filterlist", "type-whitelist", "type-elemhide", "type-elemhideexception", "type-invalid"];
-      let boolAtoms = ["selected", "dummy", "slow", "disabled"];
-
-      this.atoms = {};
-      for each (let atom in stringAtoms)
-        this.atoms[atom] = atomService.getAtom(atom);
-      for each (let atom in boolAtoms)
-      {
-        this.atoms[atom + "-true"] = atomService.getAtom(atom + "-true");
-        this.atoms[atom + "-false"] = atomService.getAtom(atom + "-false");
-      }
-
-      let columns = this.boxObject.columns;
-      for (let i = 0; i < columns.length; i++)
-        if (columns[i].element.hasAttribute("sortDirection"))
-          this.sortBy(columns[i].id, columns[i].element.getAttribute("sortDirection"));
-
-      this.refresh(true);
+      this.atoms[atom + "-true"] = atomService.getAtom(atom + "-true");
+      this.atoms[atom + "-false"] = atomService.getAtom(atom + "-false");
     }
+
+    let columns = this.boxObject.columns;
+    for (let i = 0; i < columns.length; i++)
+      if (columns[i].element.hasAttribute("sortDirection"))
+        this.sortBy(columns[i].id, columns[i].element.getAttribute("sortDirection"));
+
+    this.refresh(true);
+
+    // Stop propagation of keypress events so that these aren't intercepted by
+    // the findbar.
+    this.treeElement.inputField.addEventListener("keypress", event => {
+      event.stopPropagation();
+    }, false);
   },
 
   selection: null,
@@ -666,44 +707,49 @@ var FilterView =
     return null;
   },
 
+  generateProperties: function(list, properties)
+  {
+    if (properties)
+    {
+      // Gecko 21 and below: we have an nsISupportsArray parameter, add atoms
+      // to that.
+      for (let i = 0; i < list.length; i++)
+        if (list[i] in this.atoms)
+          properties.AppendElement(this.atoms[list[i]]);
+      return null;
+    }
+    else
+    {
+      // Gecko 22+: no parameter, just return a string
+      return list.join(" ");
+    }
+  },
+
   getColumnProperties: function(col, properties)
   {
-    col = col.id;
-
-    if (col in this.atoms)
-      properties.AppendElement(this.atoms[col]);
+    return this.generateProperties(["col-" + col.id], properties);
   },
 
   getRowProperties: function(row, properties)
   {
     if (row < 0 || row >= this.data.length)
-      return;
+      return "";
 
+    let list = [];
     let filter = this.data[row].filter;
-    properties.AppendElement(this.atoms["selected-" + this.selection.isSelected(row)]);
-    properties.AppendElement(this.atoms["slow-" + (filter instanceof RegExpFilter && defaultMatcher.isSlowFilter(filter))]);
+    list.push("selected-" + this.selection.isSelected(row));
+    list.push("slow-" + (filter instanceof RegExpFilter && defaultMatcher.isSlowFilter(filter)));
     if (filter instanceof ActiveFilter)
-      properties.AppendElement(this.atoms["disabled-" + filter.disabled]);
-    properties.AppendElement(this.atoms["dummy-" + ("dummy" in filter)]);
+      list.push("disabled-" + filter.disabled);
+    list.push("dummy-" + ("dummy" in filter));
+    list.push("type-" + filter.type);
 
-    if (filter instanceof CommentFilter)
-      properties.AppendElement(this.atoms["type-comment"]);
-    else if (filter instanceof BlockingFilter)
-      properties.AppendElement(this.atoms["type-filterlist"]);
-    else if (filter instanceof WhitelistFilter)
-      properties.AppendElement(this.atoms["type-whitelist"]);
-    else if (filter instanceof ElemHideFilter)
-      properties.AppendElement(this.atoms["type-elemhide"]);
-    else if (filter instanceof ElemHideException)
-      properties.AppendElement(this.atoms["type-elemhideexception"]);
-    else if (filter instanceof InvalidFilter)
-      properties.AppendElement(this.atoms["type-invalid"]);
+    return this.generateProperties(list, properties);
   },
 
   getCellProperties: function(row, col, properties)
   {
-    this.getColumnProperties(col, properties);
-    this.getRowProperties(row, properties);
+    return this.getRowProperties(row, properties) + " " + this.getColumnProperties(col, properties);
   },
 
   cycleHeader: function(col)
@@ -786,20 +832,20 @@ var FilterView =
       filter.disabled = !filter.disabled;
   },
 
-  isContainer: function(row) false,
-  isContainerOpen: function(row) false,
-  isContainerEmpty: function(row) true,
-  getLevel: function(row) 0,
-  getParentIndex: function(row) -1,
-  hasNextSibling: function(row, afterRow) false,
-  toggleOpenState: function(row) {},
-  getProgressMode: function() null,
-  getImageSrc: function() null,
-  isSeparator: function() false,
-  performAction: function() {},
-  performActionOnRow: function() {},
-  performActionOnCell: function() {},
-  getCellValue: function() null,
-  setCellValue: function() {},
-  selectionChanged: function() {},
+  isContainer: row => false,
+  isContainerOpen: row => false,
+  isContainerEmpty: row => true,
+  getLevel: row => 0,
+  getParentIndex: row => -1,
+  hasNextSibling: (row, afterRow) => false,
+  toggleOpenState: row => {},
+  getProgressMode: () => null,
+  getImageSrc: () => null,
+  isSeparator: () => false,
+  performAction: () => {},
+  performActionOnRow: () => {},
+  performActionOnCell: () => {},
+  getCellValue: () => null,
+  setCellValue: () => {},
+  selectionChanged: () => {}
 };
