@@ -1,7 +1,18 @@
 /*
- * This Source Code is subject to the terms of the Mozilla Public License
- * version 2.0 (the "License"). You can obtain a copy of the License at
- * http://mozilla.org/MPL/2.0/.
+ * This file is part of Adblock Plus <https://adblockplus.org/>,
+ * Copyright (C) 2006-2017 eyeo GmbH
+ *
+ * Adblock Plus is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * Adblock Plus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -38,7 +49,7 @@ var SubscriptionActions =
     let node = null;
     let tabIndex = -1;
     let subscriptions = filter.subscriptions.slice();
-    subscriptions.sort(function(s1, s2) s1.disabled - s2.disabled);
+    subscriptions.sort((s1, s2) => s1.disabled - s2.disabled);
     for (let i = 0; i < subscriptions.length; i++)
     {
       let subscription = subscriptions[i];
@@ -57,7 +68,7 @@ var SubscriptionActions =
         node.parentNode.selectItem(node);
         if (!FilterActions.visible)
           E("subscription-showHideFilters-command").doCommand();
-        Utils.runAsync(FilterView.selectFilter, FilterView, filter);
+        Utils.runAsync(() => FilterView.selectFilter(filter));
       });
     }
   },
@@ -99,7 +110,7 @@ var SubscriptionActions =
   {
     let data = Templater.getDataForNode(node || this.selectedItem);
     if (data && data.subscription instanceof DownloadableSubscription)
-      Synchronizer.execute(data.subscription, true, true);
+      Synchronizer.execute(data.subscription, true);
   },
 
   /**
@@ -111,7 +122,7 @@ var SubscriptionActions =
     {
       let subscription = FilterStorage.subscriptions[i];
       if (subscription instanceof DownloadableSubscription)
-        Synchronizer.execute(subscription, true, true);
+        Synchronizer.execute(subscription, true);
     }
   },
 
@@ -225,7 +236,7 @@ var SubscriptionActions =
       else if (accelKey == Ci.nsIDOMKeyEvent.DOM_VK_ALT)
         result = this._altMask;
     } catch(e) {}
-    this.__defineGetter__("_accelMask", function() result);
+    Object.defineProperty(this, "_accelMask", {value: result});
     return result;
   },
 
@@ -284,7 +295,7 @@ var SubscriptionActions =
 
     event.dataTransfer.addElement(node);
     event.dataTransfer.setData("text/x-moz-url", data.subscription.url);
-    event.dataTransfer.setData("text/plain", data.subscription.title);
+    event.dataTransfer.setData("text/plain", getSubscriptionTitle(data.subscription));
     this.dragSubscription = data.subscription;
     event.stopPropagation();
   },
@@ -342,9 +353,12 @@ var SubscriptionActions =
         let lines = event.dataTransfer.getData("text/plain").replace(/\r/g, "").split("\n");
         for (let i = 0; i < lines.length; i++)
         {
-          let filter = Filter.fromText(lines[i]);
-          if (filter)
+          let line = Filter.normalize(lines[i]);
+          if (line)
+          {
+            let filter = Filter.fromText(line);
             FilterStorage.addFilter(filter, data.subscription);
+          }
         }
         FilterActions.removeDraggedFilters();
         event.stopPropagation();
@@ -408,7 +422,7 @@ var TitleEditor =
 
     subscriptionNode.getElementsByClassName("titleBox")[0].selectedIndex = 1;
     let editor = subscriptionNode.getElementsByClassName("titleEditor")[0];
-    editor.value = subscription.title;
+    editor.value = getSubscriptionTitle(subscription);
     editor.setSelectionRange(0, editor.value.length);
     this.subscriptionEdited = subscriptionNode;
     editor.focus();
@@ -430,7 +444,7 @@ var TitleEditor =
     if (save)
     {
       newTitle = subscriptionNode.getElementsByClassName("titleEditor")[0].value;
-      newTitle = newTitle.replace(/^\s+/, "").replace(/\s+$/, "");
+      newTitle = newTitle.trim();
     }
 
     let subscription = Templater.getDataForNode(subscriptionNode).subscription
@@ -524,7 +538,7 @@ var SelectSubscription =
       // Show panel and focus list
       let position = (Utils.versionComparator.compare(Utils.platformVersion, "2.0") < 0 ? "after_end" : "bottomcenter topleft");
       panel.openPopup(E("selectSubscriptionButton"), position, 0, 0, false, false, event);
-      Utils.runAsync(list.focus, list);
+      Utils.runAsync(() => list.focus());
     };
     request.send();
   },
