@@ -1,7 +1,18 @@
 /*
- * This Source Code is subject to the terms of the Mozilla Public License
- * version 2.0 (the "License"). You can obtain a copy of the License at
- * http://mozilla.org/MPL/2.0/.
+ * This file is part of Adblock Plus <https://adblockplus.org/>,
+ * Copyright (C) 2006-2017 eyeo GmbH
+ *
+ * Adblock Plus is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * Adblock Plus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -76,14 +87,11 @@ ListManager.prototype =
     let subscriptions = FilterStorage.subscriptions.filter(this._filter, this);
     if (subscriptions.length)
     {
-      for each (let subscription in subscriptions)
+      for (let subscription of subscriptions)
         this.addSubscription(subscription, null);
 
       // Make sure first list item is selected after list initialization
-      Utils.runAsync(function()
-      {
-        this._list.selectItem(this._list.getItemAtIndex(this._list.getIndexOfFirstVisibleRow()));
-      }, this);
+      Utils.runAsync(() => this._list.selectItem(this._list.getItemAtIndex(this._list.getIndexOfFirstVisibleRow())));
     }
 
     this._deck.selectedIndex = (subscriptions.length ? 1 : 0);
@@ -105,7 +113,8 @@ ListManager.prototype =
       subscription: subscription,
       isExternal: subscription instanceof ExternalSubscription,
       downloading: Synchronizer.isExecuting(subscription.url),
-      disabledFilters: disabledFilters
+      disabledFilters: disabledFilters,
+      upgradeRequired: ListManager.isUpgradeRequired(subscription)
     });
     if (insertBefore)
       this._list.insertBefore(node, insertBefore);
@@ -159,14 +168,14 @@ ListManager.prototype =
   _onChange: function(action, item, param1, param2)
   {
     if ((action == "subscription.added" || action == "subscription.removed") && item.url == Prefs.subscriptions_exceptionsurl)
-      E("acceptableAds").checked = FilterStorage.subscriptions.some(function(s) s.url == Prefs.subscriptions_exceptionsurl);
+      E("acceptableAds").checked = FilterStorage.subscriptions.some(s => s.url == Prefs.subscriptions_exceptionsurl);
 
     if (action == "filter.disabled")
     {
       if (this._scheduledUpdateDisabled == null)
       {
-        this._scheduledUpdateDisabled = {__proto__: null};
-        Utils.runAsync(this.updateDisabled, this);
+        this._scheduledUpdateDisabled = Object.create(null);
+        Utils.runAsync(() => this.updateDisabled());
       }
       for (let i = 0; i < item.subscriptions.length; i++)
         this._scheduledUpdateDisabled[item.subscriptions[i].url] = true;
@@ -240,11 +249,14 @@ ListManager.prototype =
       case "subscription.homepage":
       case "subscription.lastDownload":
       case "subscription.downloadStatus":
+      case "subscription.downloading":
       {
         let subscriptionNode = Templater.getNodeForData(this._list, "subscription", item);
         if (subscriptionNode)
         {
-          Templater.getDataForNode(subscriptionNode).downloading = Synchronizer.isExecuting(item.url);
+          let data = Templater.getDataForNode(subscriptionNode);
+          data.downloading = Synchronizer.isExecuting(item.url);
+          data.upgradeRequired = ListManager.isUpgradeRequired(item);
           Templater.update(this._template, subscriptionNode);
 
           if (!document.commandDispatcher.focusedElement)
@@ -262,8 +274,8 @@ ListManager.prototype =
       {
         if (this._scheduledUpdateDisabled == null)
         {
-          this._scheduledUpdateDisabled = {__proto__: null};
-          Utils.runAsync(this.updateDisabled, this);
+          this._scheduledUpdateDisabled = Object.create(null);
+          Utils.runAsync(() => this.updateDisabled());
         }
         this._scheduledUpdateDisabled[item.url] = true;
         break;
@@ -279,13 +291,13 @@ ListManager.init = function()
 {
   new ListManager(E("subscriptions"),
                   E("subscriptionTemplate"),
-                  function(s) s instanceof RegularSubscription && !(ListManager.acceptableAdsCheckbox && s.url == Prefs.subscriptions_exceptionsurl),
+                  s => s instanceof RegularSubscription && !(ListManager.acceptableAdsCheckbox && s.url == Prefs.subscriptions_exceptionsurl),
                   SubscriptionActions.updateCommands);
   new ListManager(E("groups"),
                   E("groupTemplate"),
-                  function(s) s instanceof SpecialSubscription,
+                  s => s instanceof SpecialSubscription,
                   SubscriptionActions.updateCommands);
-  E("acceptableAds").checked = FilterStorage.subscriptions.some(function(s) s.url == Prefs.subscriptions_exceptionsurl);
+  E("acceptableAds").checked = FilterStorage.subscriptions.some(s => s.url == Prefs.subscriptions_exceptionsurl);
   E("acceptableAds").parentNode.hidden = !ListManager.acceptableAdsCheckbox;
 };
 
@@ -316,4 +328,19 @@ ListManager.allowAcceptableAds = function(/**Boolean*/ allow)
     FilterStorage.removeSubscription(subscription);
 };
 
+/**
+ * Checks whether Adblock Plus needs to be upgraded in order to support filters
+ * in a particular subscription.
+ */
+ListManager.isUpgradeRequired = function(/**Subscription*/ subscription)
+{
+  if (subscription instanceof DownloadableSubscription && subscription.requiredVersion)
+  {
+    let {addonVersion} = require("info");
+    if (Services.vc.compare(subscription.requiredVersion, addonVersion) > 0)
+      return true;
+  }
+  return false;
+};
+
 window.addEventListener("load", ListManager.init, false);
