@@ -1,7 +1,18 @@
 /*
- * This Source Code is subject to the terms of the Mozilla Public License
- * version 2.0 (the "License"). You can obtain a copy of the License at
- * http://mozilla.org/MPL/2.0/.
+ * This file is part of Adblock Plus <https://adblockplus.org/>,
+ * Copyright (C) 2006-2017 eyeo GmbH
+ *
+ * Adblock Plus is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * Adblock Plus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 Cu.import("resource://gre/modules/FileUtils.jsm");
@@ -159,7 +170,7 @@ function addSubscriptions(list, parent, level, parentTitle, parentURL)
       title.setAttribute("flex", "1");
       title.style.marginLeft = (20 * level) + "px";
       item.appendChild(title);
-  
+
       let variantTitle = document.createElement("description");
       variantTitle.setAttribute("class", "variant");
       variantTitle.textContent = variant.getAttribute("title");
@@ -206,15 +217,16 @@ function setCustomSubscription(title, url, mainSubscriptionTitle, mainSubscripti
     let link = document.createElement("label");
     link.className = "text-link";
     link.setAttribute("tooltiptext", mainSubscriptionURL);
-    link.addEventListener("click", function() UI.loadInBrowser(mainSubscriptionURL), false);
+    link.addEventListener("click", () => UI.loadInBrowser(mainSubscriptionURL), false);
     link.textContent = mainSubscriptionTitle;
     messageElement.appendChild(link);
     messageElement.appendChild(document.createTextNode(afterLink));
-    
+
     addMainCheckbox.value = mainSubscriptionURL;
     addMainCheckbox.setAttribute("_mainSubscriptionTitle", mainSubscriptionTitle)
-    addMainCheckbox.label = addMainCheckbox.getAttribute("_labelTemplate").replace(/\?1\?/g, mainSubscriptionTitle);
-    addMainCheckbox.accessKey = addMainCheckbox.accessKey;
+    let [label, accesskey] = Utils.splitLabel(addMainCheckbox.getAttribute("_labelTemplate"));
+    addMainCheckbox.label = label.replace(/\?1\?/g, mainSubscriptionTitle);
+    addMainCheckbox.accessKey = accesskey;
   }
   else
   {
@@ -227,7 +239,7 @@ function validateURL(url)
 {
   if (!url)
     return null;
-  url = url.replace(/^\s+/, "").replace(/\s+$/, "");
+  url = url.trim();
 
   // Is this a file path?
   try {
@@ -254,7 +266,7 @@ function addSubscription()
     return false;
   }
 
-  let title = E("title").value.replace(/^\s+/, "").replace(/\s+$/, "");
+  let title = E("title").value.trim();
   if (!title)
     title = url;
 
@@ -292,5 +304,5 @@ function doAddSubscription(/**String*/ url, /**String*/ title)
 
 function hasSubscription(url)
 {
-  return FilterStorage.subscriptions.some(function(subscription) subscription instanceof DownloadableSubscription && subscription.url == url);
+  return FilterStorage.subscriptions.some(subscription => subscription instanceof DownloadableSubscription && subscription.url == url);
 }
