@@ -1,7 +1,18 @@
 /*
- * This Source Code is subject to the terms of the Mozilla Public License
- * version 2.0 (the "License"). You can obtain a copy of the License at
- * http://mozilla.org/MPL/2.0/.
+ * This file is part of Adblock Plus <https://adblockplus.org/>,
+ * Copyright (C) 2006-2017 eyeo GmbH
+ *
+ * Adblock Plus is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * Adblock Plus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -36,18 +47,6 @@ let Utils = exports.Utils =
     return addonVersion;
   },
 
-  /**
-   * Returns whether we are running in Fennec, for Fennec-specific hacks
-   * @type Boolean
-   */
-  get isFennec()
-  {
-    let {application} = require("info");
-    let result = (application == "fennec" || application == "fennec2");
-    Utils.__defineGetter__("isFennec", function() result);
-    return result;
-  },
-
   /**
    * Returns the user interface locale selected for adblockplus chrome package.
    */
@@ -62,8 +61,8 @@ let Utils = exports.Utils =
     {
       Cu.reportError(e);
     }
-    Utils.__defineGetter__("appLocale", function() locale);
-    return Utils.appLocale;
+    Object.defineProperty(this, "appLocale", {value: locale});
+    return locale;
   },
 
   /**
@@ -72,13 +71,13 @@ let Utils = exports.Utils =
   get platformVersion()
   {
     let platformVersion = Services.appinfo.platformVersion;
-    Utils.__defineGetter__("platformVersion", function() platformVersion);
-    return Utils.platformVersion;
+    Object.defineProperty(this, "platformVersion", {value: platformVersion});
+    return platformVersion;
   },
 
   /**
    * Retrieves a string from global.properties string bundle, will throw if string isn't found.
-   * 
+   *
    * @param {String} name  string name
    * @return {String}
    */
@@ -95,7 +94,7 @@ let Utils = exports.Utils =
 
   /**
    * Shows an alert message like window.alert() but with a custom title.
-   * 
+   *
    * @param {Window} parentWindow  parent window of the dialog (can be null)
    * @param {String} message  message to be displayed
    * @param {String} [title]  dialog title, default title will be used if omitted
@@ -109,7 +108,7 @@ let Utils = exports.Utils =
 
   /**
    * Asks the user for a confirmation like window.confirm() but with a custom title.
-   * 
+   *
    * @param {Window} parentWindow  parent window of the dialog (can be null)
    * @param {String} message  message to be displayed
    * @param {String} [title]  dialog title, default title will be used if omitted
@@ -130,10 +129,10 @@ let Utils = exports.Utils =
   {
     if ("ownerDocument" in node && node.ownerDocument)
       node = node.ownerDocument;
-  
+
     if ("defaultView" in node)
       return node.defaultView;
-  
+
     return null;
   },
 
@@ -202,45 +201,18 @@ let Utils = exports.Utils =
 
   /**
    * Posts an action to the event queue of the current thread to run it
-   * asynchronously. Any additional parameters to this function are passed
-   * as parameters to the callback.
+   * asynchronously.
+   * @param {function} callback
    */
-  runAsync: function(/**Function*/ callback, /**Object*/ thisPtr)
+  runAsync: function(callback)
   {
-    let params = Array.prototype.slice.call(arguments, 2);
-    let runnable = {
-      run: function()
-      {
-        callback.apply(thisPtr, params);
-      }
-    };
-    Utils.threadManager.currentThread.dispatch(runnable, Ci.nsIEventTarget.DISPATCH_NORMAL);
-  },
-
-  /**
-   * Gets the DOM window associated with a particular request (if any).
-   */
-  getRequestWindow: function(/**nsIChannel*/ channel) /**nsIDOMWindow*/
-  {
-    try
-    {
-      if (channel.notificationCallbacks)
-        return channel.notificationCallbacks.getInterface(Ci.nsILoadContext).associatedWindow;
-    } catch(e) {}
-  
-    try
-    {
-      if (channel.loadGroup && channel.loadGroup.notificationCallbacks)
-        return channel.loadGroup.notificationCallbacks.getInterface(Ci.nsILoadContext).associatedWindow;
-    } catch(e) {}
-
-    return null;
+    Services.tm.currentThread.dispatch(callback, Ci.nsIEventTarget.DISPATCH_NORMAL);
   },
 
   /**
    * Generates filter subscription checksum.
    *
-   * @param {Array of String} lines filter subscription lines (with checksum line removed)
+   * @param {string[]} lines filter subscription lines (with checksum line removed)
    * @return {String} checksum or null
    */
   generateChecksum: function(lines)
@@ -250,11 +222,11 @@ let Utils = exports.Utils =
     {
       // Checksum is an MD5 checksum (base64-encoded without the trailing "=") of
       // all lines in UTF-8 without the checksum line, joined with "\n".
-  
+
       let converter = Cc["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Ci.nsIScriptableUnicodeConverter);
       converter.charset = "UTF-8";
       stream = converter.convertToInputStream(lines.join("\n"));
-  
+
       let hashEngine = Cc["@mozilla.org/security/hash;1"].createInstance(Ci.nsICryptoHash);
       hashEngine.init(hashEngine.MD5);
       hashEngine.updateFromStream(stream, stream.available());
@@ -304,7 +276,7 @@ let Utils = exports.Utils =
       return null;
 
     let appLocale = Utils.appLocale;
-    for each (let prefix in prefixes.split(/,/))
+    for (let prefix of prefixes.split(/,/))
       if (new RegExp("^" + prefix + "\\b").test(appLocale))
         return prefix;
 
@@ -373,8 +345,12 @@ let Utils = exports.Utils =
 
   /**
    * Verifies RSA signature. The public key and signature should be base64-encoded.
+   * @param {string} key
+   * @param {string} signature
+   * @param {string} data
+   * @return {boolean}
    */
-  verifySignature: function(/**String*/ key, /**String*/ signature, /**String*/ data) /**Boolean*/
+  verifySignature: function(key, signature, data)
   {
     if (!Utils.crypto)
       return false;
@@ -436,6 +412,71 @@ let Utils = exports.Utils =
       if (context && !context.isNull())
         Utils.crypto.VFY_DestroyContext(context, true);
     }
+  },
+
+  /**
+   * Returns the documentation link from the preferences.
+   */
+  getDocLink: function(/**String*/ linkID)
+  {
+    let {Prefs} = require("prefs");
+    let docLink = Prefs.documentation_link;
+    return docLink.replace(/%LINK%/g, linkID).replace(/%LANG%/g, Utils.appLocale);
+  },
+
+  /**
+   * Splits up a combined label into the label and access key components.
+   *
+   * @return {Array} An array with two strings: label and access key
+   */
+  splitLabel: function(/**String*/ label)
+  {
+    let match = /^(.*)\s*\(&(.)\)\s*(\u2026?)$/.exec(label);
+    if (match)
+    {
+      // Access key not part of the label
+      return [match[1] + match[3], match[2]];
+    }
+    else
+    {
+      // Access key part of the label
+      let pos = label.indexOf("&");
+      if (pos >= 0 && pos < label.length - 1)
+        return [label.substr(0, pos) + label.substr(pos + 1), label[pos + 1]];
+      else
+        return [label, ""];
+    }
+  },
+
+  /**
+   * Split all labels starting from a particular DOM node.
+   */
+  splitAllLabels: function(/**DOMNode*/ root)
+  {
+    let attrMap = {
+      __proto__: null,
+      "label": "value",
+      "setting": "title"
+    };
+
+    let elements = root.querySelectorAll("*[label], label[value], setting[title]");
+    for (let i = 0; i < elements.length; i++)
+    {
+      let element = elements[i];
+      let attr = (element.localName in attrMap ? attrMap[element.localName] : "label");
+      let origLabel = element.getAttribute(attr);
+
+      let [label, accesskey] = this.splitLabel(origLabel);
+      if (label != origLabel)
+        element.setAttribute(attr, label);
+      if (accesskey != "")
+        element.setAttribute("accesskey", accesskey);
+
+      // Labels forward changes of the accessKey property to their control, only
+      // set it for actual controls.
+      if (element.localName != "label")
+        element.accessKey = accesskey;
+    }
   }
 };
 
@@ -447,7 +488,7 @@ let Utils = exports.Utils =
 function Cache(/**Integer*/ size)
 {
   this._ringBuffer = new Array(size);
-  this.data = {__proto__: null};
+  this.data = Object.create(null);
 }
 exports.Cache = Cache;
 
@@ -501,7 +542,7 @@ Cache.prototype =
   clear: function()
   {
     this._ringBuffer = new Array(this._ringBuffer.length);
-    this.data = {__proto__: null};
+    this.data = Object.create(null);
   }
 }
 
@@ -509,7 +550,6 @@ Cache.prototype =
 
 XPCOMUtils.defineLazyServiceGetter(Utils, "categoryManager", "@mozilla.org/categorymanager;1", "nsICategoryManager");
 XPCOMUtils.defineLazyServiceGetter(Utils, "ioService", "@mozilla.org/network/io-service;1", "nsIIOService");
-XPCOMUtils.defineLazyServiceGetter(Utils, "threadManager", "@mozilla.org/thread-manager;1", "nsIThreadManager");
 XPCOMUtils.defineLazyServiceGetter(Utils, "promptService", "@mozilla.org/embedcomp/prompt-service;1", "nsIPromptService");
 XPCOMUtils.defineLazyServiceGetter(Utils, "effectiveTLD", "@mozilla.org/network/effective-tld-service;1", "nsIEffectiveTLDService");
 XPCOMUtils.defineLazyServiceGetter(Utils, "netUtils", "@mozilla.org/network/util;1", "nsINetUtil");
@@ -521,8 +561,6 @@ XPCOMUtils.defineLazyServiceGetter(Utils, "windowWatcher", "@mozilla.org/embedco
 XPCOMUtils.defineLazyServiceGetter(Utils, "chromeRegistry", "@mozilla.org/chrome/chrome-registry;1", "nsIXULChromeRegistry");
 XPCOMUtils.defineLazyServiceGetter(Utils, "systemPrincipal", "@mozilla.org/systemprincipal;1", "nsIPrincipal");
 XPCOMUtils.defineLazyServiceGetter(Utils, "dateFormatter", "@mozilla.org/intl/scriptabledateformat;1", "nsIScriptableDateFormat");
-XPCOMUtils.defineLazyServiceGetter(Utils, "childMessageManager", "@mozilla.org/childprocessmessagemanager;1", "nsISyncMessageSender");
-XPCOMUtils.defineLazyServiceGetter(Utils, "parentMessageManager", "@mozilla.org/parentprocessmessagemanager;1", "nsIFrameMessageManager");
 XPCOMUtils.defineLazyServiceGetter(Utils, "httpProtocol", "@mozilla.org/network/protocol;1?name=http", "nsIHttpProtocolHandler");
 XPCOMUtils.defineLazyServiceGetter(Utils, "clipboard", "@mozilla.org/widget/clipboard;1", "nsIClipboard");
 XPCOMUtils.defineLazyServiceGetter(Utils, "clipboardHelper", "@mozilla.org/widget/clipboardhelper;1", "nsIClipboardHelper");
@@ -530,9 +568,25 @@ XPCOMUtils.defineLazyGetter(Utils, "crypto", function()
 {
   try
   {
-    let ctypes = Components.utils.import("resource://gre/modules/ctypes.jsm", null).ctypes;
+    let ctypes = Cu.import("resource://gre/modules/ctypes.jsm", null).ctypes;
 
-    let nsslib = ctypes.open(ctypes.libraryName("nss3"));
+    let nsslib;
+    try
+    {
+      nsslib = ctypes.open(ctypes.libraryName("nss3"));
+    }
+    catch (e)
+    {
+      // It seems that on Mac OS X the full path name needs to be specified
+      let file;
+      // Gecko 35 added GreBinD key, see https://bugzilla.mozilla.org/show_bug.cgi?id=1077099
+      if (Services.dirsvc.has("GreBinD"))
+        file = Services.dirsvc.get("GreBinD", Ci.nsILocalFile);
+      else
+        file = Services.dirsvc.get("GreD", Ci.nsILocalFile);
+      file.append(ctypes.libraryName("nss3"));
+      nsslib = ctypes.open(file.path);
+    }
 
     let result = {};
 
